def newton_raphson(func, func_deriv, init_guess, tol=1e-6, max_iter=100):
    """Use Newton-Raphson to find the root of a function."""
    x = init_guess
    iteration = 0

    while iteration < max_iter:
        f_x = func(x)
        f_prime_x = func_deriv(x)

        if abs(f_prime_x) < tol:
            raise ValueError("Derivative is too close to zero")

        x_new = x - f_x / f_prime_x

        if abs(x_new - x) < tol:
            return x_new

        x = x_new
        print(f"Iteration {iteration}: x = {x}")
        iteration += 1

    raise Exception("Newton-Raphson did not converge")

# Define the function and its derivative
def f(x):
    return x**2 - 4

def f_derivative(x):
    return 2 * x

# Call the Newton-Raphson function to find the root
root = newton_raphson(f, f_derivative, init_guess=1.5)
print("Approximate root:", root)

